function G = Algorithm_builder(p)
h = p.h; % h
hr2 = h*sqrt(2); %h*sqrt(2)
Dirs_4 = int64([1 0 -1 0; 0 1 0 -1]); % 4 directions
Dirs_8 = int64([1 0 -1 0 1 -1 -1 1; 0 1 0 -1 1 1 -1 -1]); % 8 directions
sectors = [8 5; 5 6; 6 7; 7 8]; % diagonals in each quadrant
G = @Algorithm;

    function [T, S, t, stop] = Algorithm(T, S, max_t)
        t = max(T(T~=Inf)); 
        stop = 0;
        % for visualization
        while t <= max_t
            S1 = find(S==1);
            
            if isempty(S1)
                stop = 1;
                t = max(T(T~= Inf));
                return
            end
       
            [t, ind] = min(T(S1)); % node i, time t
            i = S1(ind);
            S(i) = 2; % add to list 2
        
            if p.g(p.x(i), t) > 0 % if not in the interior of the obstacle
                for j = p.sinv(p.s(i) + Dirs_4) % for each 4-neighbour j
                    if S(j)~=2 % if not in list 2
                        S(j) = 1; % add to list 1
                        Vj = p.sinv(p.s(j) + Dirs_8); % neighbours of j, Vj
                        TVj = T(Vj); % 8 times around j
                        TVj(S(Vj) ~= 2) = Inf; % list 2
                        TVj(TVj + hr2 < t) = Inf; % time filtering if too far into the past
                        temp = Local_Solver(TVj); % value assigned by local solver
                        if temp < T(j) 
                            T(j) = temp; % substitute only if smaller
                        end
                    end
                end
            end
        end

        function t = Local_Solver(T)
            % Calculate the time for each quadrant. Choose the least.
            t = min(arrayfun(@(sec_num) find_sector_t(sec_num), 1:4));
            
            function t = find_sector_t(sec_num)
                T1 = T(sec_num); % Node at distance 1
                T2 = min(T(sectors(sec_num, :))); % Diagonal with least time

                if T1 == Inf && T2 == Inf
                    t = Inf;
                else
                    D = (T1-T2)/h;
                    if D <= 0 % T2 too high
                        t = T1 + h;
                    elseif D * sqrt(2) >= 1 % T1 too high
                        % equivalent to: D >= sqrt(2)/2
                        t = T2 + hr2;
                    else % Sethian-Vladimirski optimal
                        t = T1 + h*sqrt(1-D^2);
                    end
                end
            end
        end

    end
end