function D = get_discontinuity_map(T, h, multiplier, mode)
    [N1, N2] = size(T);
    D = zeros(N1, N2);

    directions = [eye(2) -eye(2)];
    dnorms = multiplier * h * vecnorm(directions);
    numdirs = length(dnorms);

    for i = 1:N1
        for j = 1:N2
            i1 = [i; j];
            t1 = T(i1(1), i1(2));
            for dind = 1:numdirs
                i2 = i1 + directions(:, dind);
                if is_in_range(i2)
                    t2 = T(i2(1), i2(2));
                    if t1 + dnorms(dind) < t2
                        if mode == 1
                            D(i1(1), i1(2)) = 1;
                        else
                            D(i2(1), i2(2)) = 1;
                        end
                    end
                end
            end
        end
    end

    function isinrange = is_in_range(i)
        isinrange = 1 <= i(1) && i(1) <= N1 && 1 <= i(2) && i(2) <= N2;
    end

end