addpath("." + filesep + "DFMM")
addpath("." + filesep + "DFMM" + filesep + "Obs_builders")
close all

%% Description of Omega, I, list of obstacles {g_i}, and a proposed Tmax
% See README

name = "Circle_and_rectangle";

Omega = [-1 1; 0 2];  % Rectangle [-1, 1] x [0, 2]

I = [0; 0]; % Initial point

oblst = { % List of obstacles g_i
    ball_still(@(t) [1-.6*t; 2-t], [.3 0; 0 .3], 2) % A moving circle
    ball_still(@(t) [-1+.4*t; 2-1.3*t], [.5 0; 0 .1], Inf) % A moving rectangle
};

Tmax = 3; % Upper bound for T among the reachable nodes


%% Definition of the continuous problem pc

pc = problem_c(name, Omega, I, oblst, Tmax);

clear("name", "Omega", "I", "Tmax", "oblst")

%% Definition of the discretized problem p

h = 0.01;
p = pc.discretize(h);

%% Execution of the algorithm. Optional visualization tools.

% p.solve()
p.solve(view_process=1)
% p.solve(view_process=1, FrameRate=4, duration=10)

%% View of the T map obtained

p.view_map()

%% T map and Virtual Obstacle

T = p.T; % T map in the discretization
VO = p.VO; % Virtual Obstacle in the discretization

x = p.one_dim_grids; % Discretization of Omega