addpath("." + filesep + "DFMM")
addpath("." + filesep + "DFMM" + filesep + "Obs_builders")
close all

%% Description of Omega, I, list of obstacles {g_i}, and a proposed Tmax
% See README

name = "RotatingDoor";

Omega = [0 2; 0 1];  % Rectangle [0, 2] x [0, 1]

I = [0; .5]; % Initial set 

oblst = { % List of obstacles g_i
    ball_trans(@(t) [1; .5], @(t) [cos(.8*t) -sin(.8*t); sin(.8*t) cos(.8*t)] * diag([.7; .05]), @(t) Inf), ...
    ball_trans(@(t) [1; .5], @(t) [cos(.8*t) -sin(.8*t); sin(.8*t) cos(.8*t)] * diag([.05; .7]), @(t) Inf)
};

Tmax = 3.5; % Upper bound for T among the reachable nodes


%% Definition of the continuous problem pc

pc = problem_c(name, Omega, I, oblst, Tmax);

clear("name", "Omega", "I", "Tmax", "oblst")

%% Definition of the discretized problem p

h = 0.01;
p = pc.discretize(h);

%% Execution of the algorithm. Optional visualization tools.

% p.solve()
p.solve(view_process=1)
% p.solve(view_process=1, FrameRate=4, duration=10)

%% View of the T map obtained

p.view_map()

%% T map and Virtual Obstacle

T = p.T; % T map in the discretization
VO = p.VO; % Virtual Obstacle in the discretization

x = p.one_dim_grids; % Discretization of Omega