%% Instructions of usage of the implementation of the Dynamic Fast Marching Method

% The definition of a dynamic problem involves a set Omega, an initial
% point I, and an obstacle function g.

% The discretization involves a choice of h.

% The algorithm provides the resulting time field and the virtual obstacle.

% Read this document together with the examples provided for guidance on
% usage.


%% Definition of the initial point

% A point in R^2.

% For example,

I = [2; 3];


%% Definition of moving obstacles

% A function g(x, t): R^2 x R^+ ---> R defines the  
% moving obstacle M_t = {x | g(x,t) < 0}.
% Example:

g = @(x, t) x(2) + max(0, 0.8*(t-1)); % defines a horizontal bar that starts moving moving at time 1

% It is preffered for plotting if g can be vectorized in x, but it is not necessary

g = @(x, t) x(2, :) + max(0, 0.8*(t-1));


% If multiple obstacles are to be defined, it suffices to construct a g_i
% for each of them and then take g(x,t) = min{g_i(x,t)}_i

%% Ball - shaped obstacles

% The function ball_still returns the function defining a ball with center
% c(t), distortion matrix M in norm p. That is,

% g(x, t) = || M \ (x-c(t)) ||_p - 1.

% For example,

c = @(t) [1 ; -1 + 2*t]; % center (1, -1+2t)
M = [1 0; 0 1]; % radius 1
p = 2; % norm 2 (circle)

g = ball_still(c, M, p); % resulting defining function for a moving circle

%% Distorted balls
% If the distortion and norm depend on time, the function ball_trans provide the same result.

% For example, 

c = @(t) [2 ; 1 - t]; % center (2, 1-t)
M = @(t) [t 0; 0 t]; % radius is t
p = @(t) 2; % norm 2 (circle)

g = ball_trans(c, M, p); % resulting defining function for a moving and growing circle
